@@ -23,7 +23,7 @@ import logging
 import os
 import shutil
 
-
+from ubuntuone.oauthdesktop.main import Login, LoginProcessor
 from ubuntuone.syncdaemon import (
     config,
     action_queue,
@@ -38,10 +38,12 @@ from ubuntuone.syncdaemon.dbus_interface import (
     DBusExposedObject,
     NM_STATE_CONNECTED,
     NM_STATE_DISCONNECTED,
+    DBUS_IFACE_AUTH_NAME,
 )
 from twisted.internet import defer
 from twisted.trial.unittest import TestCase as TwistedTestCase
 from zope.interface import implements
+from oauth import oauth
 
 class FakeOAuthClient(object):
     """ Fake OAuthClient"""
@@ -49,7 +51,7 @@ class FakeOAuthClient(object):
     def __init__(self, realm):
         """ create the instance. """
         self.realm = realm
-        self.consumer = 'ubuntuone'
+        self.consumer = oauth.OAuthConsumer('ubuntuone', 'hammertime')
 
     def get_access_token(self):
         """ returns a Token"""
@@ -253,7 +255,7 @@ class DBusTwistedTestCase(BaseTwistedTestCase):
 
     def tearDown(self):
         """ Cleanup the test. """
-        d = self.cleanup_signal_receivers()
+        d = self.cleanup_signal_receivers(self.signal_receivers)
         d.addBoth(self._tearDown)
         d.addBoth(lambda _: BaseTwistedTestCase.tearDown(self))
         return d
@@ -276,10 +278,10 @@ class DBusTwistedTestCase(BaseTwistedTestCase):
         """ default error handler for DBus calls. """
         self.fail(error)
 
-    def cleanup_signal_receivers(self):
+    def cleanup_signal_receivers(self, signal_receivers):
         """ cleanup self.signal_receivers and returns a deferred """
         deferreds = []
-        for match in self.signal_receivers:
+        for match in signal_receivers:
             d = defer.Deferred()
             def callback(*args):
                 """ callback that accepts *args. """
@@ -305,6 +307,7 @@ class FakeActionQueue(object):
         """ Creates the instance """
         self.eq = self.event_queue = eq
         self.client = action_queue.ActionQueueProtocol()
+        self.client.disconnect = lambda: None
         self.uploading = {}
         self.downloading = {}
         # pylint: disable-msg=C0103
@@ -482,3 +485,59 @@ class FakeVolumeManager(object):
         """
         Do nothing
         """
+
+# OAuth stubs
+class FakeLoginProcessor(LoginProcessor):
+    """Stub login processor."""
+
+    def __init__(self, dbus_object):
+        """Initialize the login processor."""
+        LoginProcessor.__init__(self, dbus_object, use_libnotify=False)
+        self.next_login_cb = None
+
+    def login(self, realm, consumer_key, error_handler=None, reply_handler=None, do_login=True):
+        """Stub, call self.next_login_cb or send NewCredentials if
+        self.next_login_cb isn't defined.
+        """
+        self.realm = str(realm)
+        self.consumer_key = str(consumer_key)
+        if self.next_login_cb:
+            cb = self.next_login_cb[0]
+            args = self.next_login_cb[1]
+            self.next_login_cb = None
+            return cb(*args)
+        else:
+            self.dbus_object.NewCredentials(realm, consumer_key)
+
+    def clear_token(self, realm, consumer_key):
+        """Stub, do nothing"""
+        pass
+
+    def next_login_with(self, callback, args=tuple()):
+        """shortcircuit the next call to login and call the specified callback.
+        callback is usually one of: self.got_token, self.got_no_token,
+        self.got_denial or self.got_error.
+        """
+        self.next_login_cb = (callback, args)
+
+
+class FakeLogin(Login):
+    """Stub Object which listens for D-Bus OAuth requests"""
+
+    def __init__(self, bus):
+        """Initiate the object."""
+        self.bus = bus
+        self.busName = dbus.service.BusName(DBUS_IFACE_AUTH_NAME, bus=self.bus)
+        # bypass the parent class __init__ as it has the path hardcoded
+        # and we can't use '/' as the path, as we are already using it
+        # for syncdaemon. pylint: disable-msg=W0233,W0231
+        dbus.service.Object.__init__(self, object_path="/oauthdesktop",
+                                     bus_name=self.busName)
+        self.processor = FakeLoginProcessor(self)
+        self.currently_authing = False
+
+    def shutdown(self):
+        """Shutdown and remove any trace from the bus"""
+        self.busName.get_bus().release_name(self.busName.get_name())
+        self.remove_from_connection()
+
