@@ -24,6 +24,7 @@ import unittest
 from ubuntuone.storageprotocol.sharersp import (
     NotifyShareHolder,
 )
+from ubuntuone.syncdaemon import dbus_interface
 from ubuntuone.syncdaemon.dbus_interface import (
     DBUS_IFACE_STATUS_NAME,
     DBUS_IFACE_EVENTS_NAME,
@@ -36,9 +37,10 @@ from ubuntuone.syncdaemon.dbus_interface import (
 from ubuntuone.syncdaemon.volume_manager import Share
 from ubuntuone.syncdaemon.tools import DBusClient
 from ubuntuone.syncdaemon import event_queue
-from ubuntuone.syncdaemon import states
+from ubuntuone.syncdaemon import states, main
 from contrib.testing.testcase import (
     DBusTwistedTestCase,
+    FakeLogin,
 )
 from twisted.internet import defer
 from ubuntuone.syncdaemon.marker import MDMarker
@@ -1297,6 +1299,116 @@ class ConfigTests(DBusTwistedTestCase):
         return d
 
 
+class TestDBusOAuth(DBusTwistedTestCase):
+    """Tests the interaction between dbus_interface and oauthdesktop"""
+
+    def setUp(self):
+        DBusTwistedTestCase.setUp(self)
+        self.oauth = FakeLogin(self.bus)
+        self._old_path = dbus_interface.DBUS_PATH_AUTH
+        dbus_interface.DBUS_PATH_AUTH = '/oauthdesktop'
+
+    def tearDown(self):
+        # collect all signal receivers registered during the test
+        signal_receivers = set()
+        with self.bus._signals_lock:
+            for group in self.bus._signal_recipients_by_object_path.values():
+                for matches in group.values():
+                    for match in matches.values():
+                        signal_receivers.update(match)
+        d = self.cleanup_signal_receivers(signal_receivers)
+        def shutdown(r):
+            self.oauth.shutdown()
+            dbus_interface.DBUS_PATH_AUTH = self._old_path
+        d.addCallback(shutdown)
+        d.addCallback(lambda _: DBusTwistedTestCase.tearDown(self))
+        return d
+
+    def test_new_credentials_signal(self):
+        """NewCredentials signal test"""
+        self.oauth.processor.next_login_with(self.oauth.processor.got_token,
+                                             ('my_token',))
+        d = self.dbus_iface._request_token()
+        def check(info):
+            realm, key = info
+            expected = (u'http://test.ubuntuone.com', u'ubuntuone')
+            self.assertEquals((realm, key), expected)
+        d.addCallbacks(check, self.fail)
+        return d
+
+    def test_no_credentials_signal(self):
+        """NoCredentials signal test"""
+        self.oauth.processor.next_login_with(self.oauth.processor.got_no_token)
+        d = self.dbus_iface._request_token()
+        def check(f):
+            self.assertEquals(f.getErrorMessage(), 'NoCredentials')
+        d.addCallbacks(self.fail, check)
+        return d
+
+    def test_auth_denied_signal(self):
+        """AuthorizationDenied signal test"""
+        self.oauth.processor.next_login_with(self.oauth.processor.got_denial)
+        d = self.dbus_iface._request_token()
+        def check(f):
+            self.assertEquals(f.getErrorMessage(), 'AuthorizationDenied')
+        d.addCallbacks(self.fail, check)
+        return d
+
+    def test_oauth_error_signal(self):
+        """OAuthError signal test"""
+        self.oauth.processor.next_login_with(self.oauth.processor.got_error,
+                                            ('error!',))
+        d = self.dbus_iface._request_token()
+        def check(f):
+            self.assertEquals(f.getErrorMessage(), 'OAuthError: error!')
+        d.addCallbacks(self.fail, check)
+        return d
+
+    def test_connect_without_token(self):
+        """Test connecting without having a token"""
+        # replace the get_access_token method in Main
+        def get_access_token():
+            raise main.NoAccessToken('no token')
+        self.main.get_access_token = get_access_token
+        self.dbus_iface.disconnect()
+        self.oauth.processor.next_login_with(self.oauth.processor.got_token,
+                                             ('the token',))
+        d = self.dbus_iface.connect()
+        def check(result):
+            self.assertEquals(result, None)
+        d.addCallbacks(check, self.fail)
+        return d
+
+    def test_connect_with_token(self):
+        """Test connecting with a token"""
+        self.dbus_iface.disconnect()
+        self.oauth.processor.next_login_with(self.oauth.processor.got_token,
+                                             ('the token',))
+        d = self.dbus_iface.connect()
+        # check that the deferred was fired
+        self.assertEquals(d.called, True)
+        self.assertEquals(d.result, None)
+        d.addCallbacks(lambda r: self.assertEquals(r, None), self.fail)
+        return d
+
+    def test_error_on_login(self):
+        """Test connecting without having a token and getting an error in the
+        call to oauthdesktop login().
+        """
+        # replace the get_access_token method in Main
+        def get_access_token():
+            raise main.NoAccessToken('no token')
+        self.main.get_access_token = get_access_token
+        self.dbus_iface.disconnect()
+        def broken_login(*args):
+            raise ValueError('oops, login is broken!')
+        self.oauth.processor.next_login_with(broken_login)
+        d = self.dbus_iface.connect()
+        def check(result):
+            self.assertIn('ValueError: oops, login is broken!', result.getErrorMessage())
+        d.addCallbacks(self.fail, check)
+        return d
+
 
 def test_suite():
     # pylint: disable-msg=C0111
