@@ -120,6 +120,10 @@ class BaseTestCase(unittest.TestCase):
 
         self.fsm.create(filepath, which_share.id, is_dir=is_dir)
         self.fsm.set_node_id(filepath, "uuid"+path)
+
+        # for files we put hashes to signal them not non-content
+        if not is_dir:
+            self.fsm.set_by_path(filepath, local_hash="h", server_hash="h")
         return filepath
 
 
@@ -892,6 +896,7 @@ class PushTests(TwistedBase):
         filepath = os.path.join(self.share.path, "a")
         self.fsm.create(filepath, self.share.id, is_dir=False)
         self.fsm.set_node_id(filepath, "uuid1")
+        self.fsm.set_by_path(filepath, local_hash="hash", server_hash="hash")
 
         def check(_):
             """ asserts that GoodListener was called. """
@@ -1131,7 +1136,7 @@ class BadStateTests(TwistedBase):
                                 mdid + ".u1partial." + os.path.basename(path))
         self.fsm.set_node_id(path, "uuid")
 
-        # start the download, never complet it
+        # start the download, never complete it
         self.fsm.set_by_mdid(mdid, server_hash="blah-hash-blah")
         self.fsm.create_partial("uuid", self.share.id)
         fh = self.fsm.get_partial_for_writing("uuid", self.share.id)
@@ -1157,7 +1162,7 @@ class BadStateTests(TwistedBase):
                                 mdid + ".u1partial." + os.path.basename(path))
         self.fsm.set_node_id(path, "uuid")
 
-        # start the download, never complet it
+        # start the download, never complete it
         self.fsm.set_by_mdid(mdid, server_hash="blah-hash-blah")
         self.fsm.create_partial("uuid", self.share.id)
         fh = self.fsm.get_partial_for_writing("uuid", self.share.id)
@@ -1188,7 +1193,7 @@ class BadStateTests(TwistedBase):
                                 mdid + ".u1partial." + os.path.basename(path))
         self.fsm.set_node_id(path, "uuid")
 
-        # start the download, never complet it
+        # start the download, never complete it
         self.fsm.set_by_mdid(mdid, server_hash="blah-hash-blah")
         self.fsm.create_partial("uuid", self.share.id)
         fh = self.fsm.get_partial_for_writing("uuid", self.share.id)
@@ -1258,7 +1263,7 @@ class BadStateTests(TwistedBase):
         mdid = self.fsm.create(path, self.share.id, is_dir=True)
         self.fsm.set_node_id(path, "uuid")
 
-        # start the download, never complet it
+        # start the download, never complete it
         self.fsm.set_by_mdid(mdid, server_hash="blah-hash-blah")
         self.fsm.create_partial("uuid", self.share.id)
         fh = self.fsm.get_partial_for_writing("uuid", self.share.id)
@@ -1404,17 +1409,85 @@ class BadStateTests(TwistedBase):
         # this is a case when a file is written with content, the make_file
         # to the server was ok (we have the file uuid), but before
         # HQ finishes everything is stopped; when started again, it needs
-        # to generate the FILE_CLOSE_WRITE, to start the hash&upload process
-        # again
+        # to generate the corresponding events, to start the hash&upload
+        # process again
         path = os.path.join(self.share.path, "a")
         self.fsm.create(path, self.share.id, is_dir=False)
         self.fsm.set_node_id(path, "uuid")
+        self.fsm.set_by_path(path, local_hash="", server_hash="")
         with open(path, "w") as fh:
             fh.write("foo")
 
         def check(_):
             '''check'''
-            self.assertEqual(self.eq.pushed, [('FS_FILE_CLOSE_WRITE', path)])
+            self.assertEqual(self.eq.pushed, [('FS_FILE_CREATE', path),
+                                              ('FS_FILE_CLOSE_WRITE', path)
+                                             ])
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_notcontent_file(self):
+        '''The file is created but never started to download.'''
+        # create the file in metadata
+        path = os.path.join(self.share.path, "a")
+#        open(path, "w").close()
+        mdid = self.fsm.create(path, self.share.id, is_dir=False, node_id="1")
+
+        def check(_):
+            '''No event, and no MD'''
+            self.assertEqual(self.eq.pushed, [])
+            self.assertFalse(self.fsm.has_metadata(path=path))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_remove_only_partial_of_childs(self):
+        '''We were downloading the file, but it was interrupted.
+        and we have a partial of another fiel in a subdir.'''
+        # create the file in metadata
+        path = os.path.join(self.share.path, "a")
+        mdid = self.fsm.create(path, self.share.id, is_dir=False)
+        partial_path = os.path.join(self.fsm.partials_dir,
+                                mdid + ".u1partial." + os.path.basename(path))
+        self.fsm.set_node_id(path, "uuid")
+
+        # start the download, never complete it
+        self.fsm.set_by_mdid(mdid, server_hash="blah-hash-blah")
+        self.fsm.create_partial("uuid", self.share.id)
+        fh = self.fsm.get_partial_for_writing("uuid", self.share.id)
+        fh.write("foobar")
+        self.assertTrue(os.path.exists(partial_path))
+
+        # create another file with a partial, but one level deeper
+        dir = os.path.join(self.share.path, "dir")
+        os.mkdir(dir)
+        path_b = os.path.join(self.share.path, "dir", "b")
+        open(path_b, "w").close()
+        mdid_dir = self.fsm.create(dir, self.share.id, is_dir=True)
+        self.fsm.set_node_id(dir, "uuid2")
+
+        mdid_b = self.fsm.create(path_b, self.share.id, is_dir=False)
+        partial_path_b = os.path.join(self.fsm.partials_dir,
+                                mdid_b + ".u1partial." + os.path.basename(path_b))
+        self.fsm.set_node_id(path_b, "uuid1")
+
+
+        # start the download, never complete it
+        self.fsm.set_by_mdid(mdid_b, server_hash="blah-hash-blah-b")
+        self.fsm.create_partial("uuid1", self.share.id)
+        fh = self.fsm.get_partial_for_writing("uuid1", self.share.id)
+        fh.write("foobar")
+        self.assertTrue(os.path.exists(partial_path_b))
+
+        def check(_):
+            '''arrange the metadata so later server_rescan will do ok'''
+            mdobj = self.fsm.get_by_mdid(mdid)
+            self.assertFalse(mdobj.info.is_partial)
+            self.assertEqual(mdobj.server_hash, mdobj.local_hash)
+            self.assertFalse(os.path.exists(partial_path))
+            self.assertFalse(os.path.exists(partial_path_b))
+            self.assertTrue(os.path.exists(path_b))
 
         self.startTest(check)
         return self.deferred
