@@ -34,6 +34,7 @@ from oauth import oauth
 from ubuntuone.clientdefs import VERSION as ubuntuone_client_version
 from ubuntuone.oauthdesktop.key_acls import set_all_key_acls
 
+from threading import Thread
 from twisted.internet import reactor
 from twisted.web import server, resource
 
@@ -55,9 +56,12 @@ NM_STATE_DISCONNECTED = 4
 def _connect_wrapper(self):
     """Override HTTPSConnection.connect to require certificate checks"""
     sock = socket.create_connection((self.host, self.port), self.timeout)
-    if self._tunnel_host:
-        self.sock = sock
-        self._tunnel()
+    try:
+        if self._tunnel_host:
+            self.sock = sock
+            self._tunnel()
+    except AttributeError:
+        pass
     self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
         cert_reqs=ssl.CERT_REQUIRED, 
         ca_certs="/etc/ssl/certs/ca-certificates.crt")
@@ -290,7 +294,14 @@ class AuthorisationClient(object):
 
             def got_error(error):
                 """Handler for D-Bus errors when calling state()."""
-                logger.error("Unable to contact NetworkManager")
+                if error.get_dbus_name() == \
+                        'org.freedesktop.DBus.Error.ServiceUnknown':
+                    logger.debug("NetworkManager not available.")
+                    self.acquire_access_token(description, store)
+                else:
+                    logger.error("Error contacting NetworkManager: %s" % \
+                                     str(error))
+
 
             iface.state(reply_handler=got_state, error_handler=got_error)
 
@@ -342,7 +353,8 @@ class AuthorisationClient(object):
         nodename = os.uname()[1]
         if nodename:
             oauth_request.set_parameter("description", nodename)
-        self.open_in_browser(oauth_request.to_url())
+        Thread(target=self.open_in_browser, name="authorization",
+               args=(oauth_request.to_url(),)).start()
 
     def get_temporary_httpd(self, nonce, retrieve_function, store):
         "A separate class so it can be mocked in testing"
