@@ -16,4 +16,8 @@
 """ client module """
 
 # required capabilities
-REQUIRED_CAPS = frozenset(["no-content", "account-info", "resumable-uploads"])
+REQUIRED_CAPS = frozenset(["no-content",
+                           "account-info",
+                           "resumable-uploads",
+                           "fix462230",
+                          ])
