@@ -44,7 +44,7 @@ from twisted.internet import threads
 
 from oauth import oauth
 from ubuntuone.storageprotocol.context import get_ssl_context
-from ubuntuone.storageprotocol import protocol_pb2
+from ubuntuone.storageprotocol import protocol_pb2, request
 from ubuntuone.storageprotocol.client import ThrottlingStorageClient, \
     ThrottlingStorageClientFactory
 from ubuntuone.syncdaemon.logger import mklog, TRACE
@@ -193,6 +193,16 @@ class ActionQueueProtocol(LoggingStorageClient):
         if self._finished:
             # the factory is no longer interested in what we have to say
             return
+
+        if self.factory.client is not None:
+            # this is a connectionMade for a second client while we still have
+            # a previous one alive: discard!
+            self._finished = True
+            logger.debug("discarding client, duplicated connectionMade")
+            if self.transport is not None:
+                self.transport.loseConnection()
+            return
+
         logger.debug("connection made")
         LoggingStorageClient.connectionMade(self)
         self.connection_state = 'connected'
@@ -975,11 +985,22 @@ class ActionQueue(ConfigurableThrottlingFactory, object):
         client = self.client
         try:
             yield client.oauth_authenticate(oauth_consumer, self.token)
+        except request.StorageRequestError, e:
+            if client is not self.client:
+                return
+            if e.error_message.error.type == \
+                                    protocol_pb2.Error.AUTHENTICATION_FAILED:
+                logger.error("OAuth failed: %s", e)
+                self.event_queue.push('SYS_OAUTH_ERROR', error=str(e))
+            else:
+                logger.error("StorageRequestError during OAuth: %s", e)
+                self.event_queue.push('SYS_UNKNOWN_ERROR')
+            self.deferred.callback(Failure(e))
         except Exception, e:
             if client is not self.client:
                 return
-            logger.error("OAuth failed: %s", e.message)
-            self.event_queue.push('SYS_OAUTH_ERROR', error=e.message)
+            logger.error("Generic error during OAuth: %s", e)
+            self.event_queue.push('SYS_UNKNOWN_ERROR')
             self.deferred.callback(Failure(e))
         else:
             if client is not self.client:
@@ -1835,7 +1856,7 @@ class CreateShare(ActionQueueCommand):
         except HTTPError, e:
             deferred.errback(Failure(e))
 
-        deferred.callback()
+        deferred.callback(None)
 
     def _run(self):
         """
