@@ -31,7 +31,6 @@ from xdg.BaseDirectory import (
     xdg_data_home,
     xdg_cache_home,
 )
-from ubuntuone.syncdaemon import logger
 
 
 CONFIG_FILE = 'syncdaemon.conf'
@@ -73,16 +72,17 @@ def log_level_parser(value):
     level = getattr(logging, value, None)
     if level is None:
         # if level don't exists in our custom levels, fallback to DEBUG
-        level = logger.levels.get(value, 'DEBUG')
+        level = getattr(logging, 'DEBUG')
     return level
 
 
 def throttling_limit_parser(value):
     """parser for throttling limit values, if value < 0 returns None"""
+    value = int(value)
     if value < 0:
         return None
     else:
-        return int(value)
+        return value
 
 
 def get_parsers():
@@ -108,7 +108,6 @@ def get_config_files():
     # reverse the list as load_config_paths returns the user dir first
     config_files.reverse()
     # if we are running from a branch, get the config file from it too
-    #le = os.path.join(os.path.split(os.path.dirname(__file__))[0],
     config_file = os.path.join('data', CONFIG_FILE)
     if os.path.exists(config_file):
         config_files.append(config_file)
@@ -124,7 +123,7 @@ def get_user_config(config_file=_user_config_path):
 
 
 def requires_section(section):
-    """decorator to enfornce the exitense of a section in the config."""
+    """decorator to enforce the existence of a section in the config."""
     def wrapper(meth):
         """the wrapper"""
         def wrapped(self, *args, **kwargs):
@@ -177,12 +176,15 @@ class _Config(TypedConfigParser):
             os.rename(self.config_file, self.config_file+'.old')
         os.rename(self.config_file+'.new', self.config_file)
 
-    def get(self, section, option):
+    def get_parsed(self, section, option):
         """custom get that fallback to our custom defaults"""
         try:
-            return super(_Config, self).get(section, option)
+            value = super(_Config, self).get(section, option)
+            # get the parser from the default config
+            default = self.default.get(section, option)
+            return default.parser(value)
         except ConfigParser.NoOptionError, e:
-            return str(self.default.get(section, option).value)
+            return self.default.get(section, option).value
 
     # throttling section get/set
     @requires_section(THROTTLING)
@@ -199,15 +201,13 @@ class _Config(TypedConfigParser):
 
     @requires_section(THROTTLING)
     def get_throttling(self):
-        return self.getboolean(THROTTLING, 'on')
+        return self.get_parsed(THROTTLING, 'on')
 
     @requires_section(THROTTLING)
     def get_throttling_read_limit(self):
-        value = self.getint(THROTTLING, 'read_limit')
-        return throttling_limit_parser(value)
+        return self.get_parsed(THROTTLING, 'read_limit')
 
     @requires_section(THROTTLING)
     def get_throttling_write_limit(self):
-        value = self.getint(THROTTLING, 'write_limit')
-        return throttling_limit_parser(value)
+        return self.get_parsed(THROTTLING, 'write_limit')
 
