@@ -16,14 +16,20 @@
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 """ DBUS interface module """
-from itertools import groupby, chain
 
 import dbus.service
+import logging
+
+from dbus import DBusException
+from itertools import groupby, chain
+
+from twisted.internet import defer
+from twisted.python.failure import Failure
 
 from ubuntuone.syncdaemon.event_queue import EVENTS
 from ubuntuone.syncdaemon.interfaces import IMarker
 from ubuntuone.syncdaemon.config import get_user_config
-import logging
+
 
 # Disable the "Invalid Name" check here, as we have lots of DBus style names
 # pylint: disable-msg=C0103
@@ -46,6 +52,9 @@ NM_STATE_DISCONNECTED = 4
 NM_STATE_EVENTS = {NM_STATE_CONNECTED: 'SYS_NET_CONNECTED',
                    NM_STATE_DISCONNECTED: 'SYS_NET_DISCONNECTED'}
 
+# OAuthDesktop constants
+DBUS_IFACE_AUTH_NAME = "com.ubuntuone.Authentication"
+DBUS_PATH_AUTH = "/"
 
 logger = logging.getLogger("ubuntuone.SyncDaemon.DBus")
 
@@ -889,12 +898,7 @@ class Config(DBusExposedObject):
                                     error_handler=None):
         """Enable bandwidth throttling."""
         try:
-            # modify and save the config file
-            config = get_user_config()
-            config.set_throttling(True)
-            config.save()
-            # modify AQ settings
-            self.dbus_iface.action_queue.enable_throttling(True)
+            self._set_throttling_enabled(True)
             if reply_handler:
                 reply_handler()
             # pylint: disable-msg=W0703
@@ -911,12 +915,7 @@ class Config(DBusExposedObject):
                                      error_handler=None):
         """Disable bandwidth throttling."""
         try:
-            # modify and save the config file
-            config = get_user_config()
-            config.set_throttling(True)
-            config.save()
-            # modify AQ settings
-            self.dbus_iface.action_queue.enable_throttling(False)
+            self._set_throttling_enabled(False)
             if reply_handler:
                 reply_handler()
             # pylint: disable-msg=W0703
@@ -926,6 +925,15 @@ class Config(DBusExposedObject):
             else:
                 raise
 
+    def _set_throttling_enabled(self, enabled):
+        """set throttling enabled value and save the config"""
+        # modify and save the config file
+        config = get_user_config()
+        config.set_throttling(enabled)
+        config.save()
+        # modify AQ settings
+        self.dbus_iface.action_queue.enable_throttling(enabled)
+
     @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
                          in_signature='', out_signature='b',
                          async_callbacks=('reply_handler', 'error_handler'))
@@ -999,6 +1007,7 @@ class DBusInterface(object):
                         'org.freedesktop.NetworkManager',
                         '/org/freedesktop/NetworkManager',
                         follow_name_owner_changes=True)
+                    iface = dbus.Interface(nm, 'org.freedesktop.NetworkManager')
                 except dbus.DBusException, e:
                     if e.get_dbus_name() == \
                         'org.freedesktop.DBus.Error.ServiceUnknown':
@@ -1009,10 +1018,9 @@ class DBusInterface(object):
                     else:
                         raise
                 else:
-                    nm.Get('org.freedesktop.NetworkManager', 'State',
-                           reply_handler=self.connection_state_changed,
-                           error_handler=error_handler,
-                           dbus_interface='org.freedesktop.DBus.Properties')
+                    iface.state(reply_handler=self.connection_state_changed,
+                                error_handler=error_handler)
+
         # register a handler to NM StateChanged signal
         self.system_bus.add_signal_receiver(self.connection_state_changed,
                                signal_name='StateChanged',
@@ -1066,14 +1074,64 @@ class DBusInterface(object):
         if event is not None:
             self.event_queue.push(event)
 
-    def connect(self):
-        """ Push the SYS_CONNECT event with the tokens in the keyring. """
+    @defer.inlineCallbacks
+    def connect(self, do_login=True):
+        """Push the SYS_CONNECT event with the token in the keyring or
+        request the token via oauthdesktop and push the acquired token."""
         from ubuntuone.syncdaemon.main import NoAccessToken
         try:
             access_token = self.main.get_access_token()
             self.event_queue.push('SYS_CONNECT', access_token)
         except NoAccessToken, e:
-            logger.exception("Can't get the auth token")
+            if do_login:
+                yield self._request_token()
+                self.connect(do_login=False)
+            else:
+                logger.exception("Can't get the auth token")
+
+    def _request_token(self):
+        """Request to OAuthDesktop to fetch the token"""
+        from ubuntuone.syncdaemon.main import NoAccessToken
+        d = defer.Deferred()
+        def error_handler(error):
+            """default dbus error handler"""
+            if not d.called:
+                d.errback(Failure(error))
+
+        def signal_handler(*args, **kwargs):
+            """Signal handler"""
+            member = kwargs.get('member', None)
+            if member in ('NoCredentials', 'AuthorizationDenied',
+                          'OAuthError'):
+                if not args:
+                    d.errback(Failure(NoAccessToken(member)))
+                else:
+                    d.errback(Failure(NoAccessToken("%s: %s" %
+                                                    (member, args[0]))))
+            elif member == 'NewCredentials' and not d.called:
+                d.callback(args)
+        # register signal handlers for each kind of error
+        match = self.bus.add_signal_receiver(signal_handler,
+                                          member_keyword='member',
+                                          dbus_interface=DBUS_IFACE_AUTH_NAME)
+        # call oauthdesktop
+        try:
+            client = self.bus.get_object(DBUS_IFACE_AUTH_NAME, DBUS_PATH_AUTH,
+                                         follow_name_owner_changes=True)
+            iface = dbus.Interface(client, DBUS_IFACE_AUTH_NAME)
+            iface.login(self.main.realm, self.main.oauth_client.consumer.key,
+                        # ignore the reply, we get the result via signals
+                        reply_handler=lambda: None,
+                        error_handler=error_handler)
+        except DBusException, e:
+            d.errback(Failure(e))
+        def remove_signal_receiver(r):
+            # cleanup the signal receivers
+            self.bus.remove_signal_receiver(match,
+                                            dbus_interface=DBUS_IFACE_AUTH_NAME)
+            return r
+        d.addBoth(remove_signal_receiver)
+        return d
 
     def disconnect(self):
         """ Push the SYS_DISCONNECT event. """
