@@ -203,6 +203,12 @@ class FileSystemManager(object):
       - idx_path: relationship path -> mdid
       - idx_node_id: relationship (share_id, node_id) -> mdid
     '''
+
+    CONFLICT_SUFFIX = '.u1conflict'
+    CHANGED_LOCAL = 'LOCAL'
+    CHANGED_SERVER = 'SERVER'
+    CHANGED_NONE = 'NONE'
+
     def __init__(self, data_dir, partials_dir, vm):
         if not isinstance(data_dir, basestring):
             raise TypeError("data_dir should be a string instead of %s" % \
@@ -651,6 +657,33 @@ class FileSystemManager(object):
             del self._idx_node_id[(mdobj["share_id"], mdobj["node_id"])]
         del self.fs[mdid]
 
+    def _delete_dir_tree(self, path):
+        """Helper function to recursively remove a non-empty directory.
+
+        Removes the dir if there aren't local changes, and returns True.
+        If there are, return False.
+
+        Notifications are disabled for every sub-path within path.
+
+        """
+        dir_deleted = False
+        subtree = self.get_paths_starting_with(path)
+
+        for p, is_dir in subtree:
+            if self.changed(path=p) == self.CHANGED_LOCAL:
+                break
+        else:
+            # no local modification in the path tree
+            for p, is_dir in subtree:
+                filter_name = is_dir and "FS_DIR_DELETE" or "FS_FILE_DELETE"
+                self.eq.add_to_mute_filter(filter_name, p)
+                if p != path:
+                    self.delete_metadata(p)
+            shutil.rmtree(path)
+            dir_deleted = True
+
+        return dir_deleted
+
     def delete_file(self, path):
         '''Deletes a file/dir and the metadata.'''
         # adjust the metadata
@@ -670,9 +703,11 @@ class FileSystemManager(object):
                     os.remove(path)
         except OSError, e:
             if e.errno == errno.ENOTEMPTY:
-                raise
-            m = "OSError %s when trying to remove file/dir %r"
-            log_warning(m, e, path)
+                if not self._delete_dir_tree(path=path):
+                    raise
+            else:
+                m = "OSError %s when trying to remove file/dir %r"
+                log_warning(m, e, path)
         self.delete_metadata(path)
 
     def move_to_conflict(self, mdid):
@@ -680,7 +715,7 @@ class FileSystemManager(object):
         mdobj = self.fs[mdid]
         path = self.get_abspath(mdobj['share_id'], mdobj['path'])
         log_debug("move_to_conflict: path=%r mdid=%r" % (path, mdid))
-        base_to_path = to_path =  path + ".u1conflict"
+        base_to_path = to_path =  path + self.CONFLICT_SUFFIX
         ind = 0
         while os.path.exists(to_path):
             ind += 1
@@ -874,7 +909,20 @@ class FileSystemManager(object):
         raise TypeError("Incorrect arguments for 'has_metadata': %r" % kwargs)
 
     def changed(self, **kwargs):
-        '''Return True if there's metadata for a given object.'''
+        """Return whether a given node has changed or not.
+
+        The node can be defined by any of the following:
+         * path
+         * metadata's id (mdid)
+         * node_id and share_id
+
+        Return:
+         * LOCAL if the node has local modifications that the server is
+         not aware of.
+         * SERVER if the node is not fully downloaded.
+         * NONE the node has not changed.
+
+        """
         # get the values
         mdid = self._get_mdid_from_args(kwargs, "changed")
         mdobj = self.fs[mdid]
@@ -888,13 +936,26 @@ class FileSystemManager(object):
                 return "We broke the Universe! local_hash %r, server_hash %r,"\
                        " is_partial %r" % (local_hash, server_hash, is_partial)
             else:
-                return 'NONE'
+                return self.CHANGED_NONE
         else:
             if is_partial:
-                return 'SERVER'
+                return self.CHANGED_SERVER
             else:
-                return 'LOCAL'
-        return
+                return self.CHANGED_LOCAL
+
+    def local_changed(self, path):
+        """Return whether a given node have locally changed or not.
+
+        Return True if the node at `path' (or any of its children) has
+        been locally modified.
+
+        """
+        has_changed = False
+        for p, is_dir in self.get_paths_starting_with(path):
+            if self.changed(path=p) == self.CHANGED_LOCAL:
+                has_changed = True
+                break
+        return has_changed
 
     def dir_content(self, path):
         '''Returns the content of the directory in a server-comparable way.'''
@@ -958,11 +1019,15 @@ class FileSystemManager(object):
         share = self._get_share(share_id)
         return EnableShareWrite(share, path)
 
-    def get_paths_starting_with(self, base_path):
+    def get_paths_starting_with(self, base_path, include_base=True):
         """ return a list of paths that starts with: path. """
+        len_func = str.__len__ # avoid lookup
+        base_length = len_func(base_path)
+
         all_paths = []
         for path in self._idx_path:
-            if path.startswith(base_path):
+            candidate = include_base or len_func(path) > base_length
+            if candidate and path.startswith(base_path):
                 mdid = self._idx_path[path]
                 mdobj = self.fs[mdid]
                 all_paths.append((path, mdobj['is_dir']))
