@@ -153,14 +153,14 @@ class LocalRescan(object):
 
         reactor.callLater(0, safe_scan)
 
-    def _get_share_dirs(self, share_id, path):
-        '''Get all the directories in a share.'''
-        all_share_dirs = []
+    def _get_share_info(self, share_id, path):
+        '''Get all the objects information for a share.'''
+        share_info = []
         for obj in self.fsm.get_mdobjs_by_share_id(share_id, path):
             changd = self.fsm.changed(mdid=obj.mdid)
-            all_share_dirs.append(
-                    (obj.path, obj.is_dir, obj.stat, changd, obj.node_id))
-        return all_share_dirs
+            share_info.append((obj.path, obj.is_dir, obj.stat, changd,
+                               obj.node_id, obj.local_hash, obj.server_hash))
+        return share_info
 
     def _scan_tree(self, share, path, scan_partials, mdid):
         '''Scans a whole tree, using the received path as root.'''
@@ -205,8 +205,8 @@ class LocalRescan(object):
         log_debug("comparing directory %r", dirpath)
 
         # get the share info
-        all_share_dirs = self._get_share_dirs(share.id, dirpath)
-        shouldbe = self._paths_filter(all_share_dirs, dirpath, len(share.path))
+        share_info = self._get_share_info(share.id, dirpath)
+        shouldbe = self._paths_filter(share_info, dirpath, len(share.path))
 
         def despair(message, fullname, also_children=False, also_remove=None):
             '''Something went very bad with this node, converge!'''
@@ -244,7 +244,7 @@ class LocalRescan(object):
         for dname in dirnames:
             fullname = os.path.join(dirpath, dname)
             if dname in shouldbe:
-                is_dir, statinfo, changed = shouldbe.pop(dname)
+                is_dir, statinfo, changed, _ = shouldbe.pop(dname)
                 if not is_dir:
                     # it's there, but it's a file!
                     log_debug("comp yield: file %r became a dir!", fullname)
@@ -277,7 +277,7 @@ class LocalRescan(object):
         for fname in filenames:
             fullname = os.path.join(dirpath, fname)
             if fname in shouldbe:
-                is_dir, statinfo, changed = shouldbe.pop(fname)
+                is_dir, statinfo, changed, _ = shouldbe.pop(fname)
                 if is_dir:
                     log_debug("comp yield: dir %r became a file!", fullname)
                     # it's there, but it's a directory!
@@ -327,7 +327,7 @@ class LocalRescan(object):
                         os.remove(fullname)
                         continue
 
-                    is_dir, statinfo, changed = shouldbe.pop(realname)
+                    is_dir, statinfo, changed, _ = shouldbe.pop(realname)
                     if is_dir:
                         m = ".partial of a file that MD says it's a dir: %r"
                         despair(m, realfullname, also_remove=fullname)
@@ -382,7 +382,7 @@ class LocalRescan(object):
             else:
                 realfullname = self.fsm.get_abspath(mdobj.share_id, mdobj.path)
 
-            if not realfullname.startswith(dirpath):
+            if os.path.dirname(realfullname) != dirpath:
                 # if this partial isn't for the current dirpath, ignore it!
                 continue
             if realfullname == dirpath:
@@ -394,7 +394,7 @@ class LocalRescan(object):
                 despair(m, realfullname, also_remove=partial_path)
                 continue
 
-            is_dir, statinfo, changed = shouldbe.pop(realname)
+            is_dir, statinfo, changed, _ = shouldbe.pop(realname)
             if is_dir:
                 m = ".partial of a file that MD says it's a dir: %r"
                 despair(m, realfullname, also_remove=partial_path)
@@ -408,10 +408,9 @@ class LocalRescan(object):
                 self.fsm.set_by_mdid(mdobj.mdid,
                                      server_hash=mdobj.local_hash)
                 self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
-                check_stat(realfullname, statinfo)
 
         # all these don't exist anymore
-        for name, (is_dir, statinfo, changed) in shouldbe.iteritems():
+        for name, (is_dir, statinfo, changed, lhash) in shouldbe.iteritems():
             fullname = os.path.join(dirpath, name)
             if is_dir:
                 if changed not in ("SERVER", "NONE"):
@@ -429,7 +428,7 @@ class LocalRescan(object):
                 to_inform = []
 
                 # get all the info inside that dir
-                for shrpath, is_dir, statinfo, _, _ in all_share_dirs:
+                for shrpath, is_dir, statinfo, _, _, _, _ in share_info:
                     if shrpath.startswith(base_path):
                         qparts = len(shrpath.split(os.path.sep))
                         to_inform.append((qparts, shrpath, is_dir))
@@ -453,11 +452,14 @@ class LocalRescan(object):
                     self.fsm.delete_metadata(fullname)
                     continue
 
-                log_debug("comp yield: file %r is gone!", fullname)
-                events.append(('FS_FILE_DELETE', fullname))
+                # if it had content somewhen, now is gone (otherwise it was
+                # never really created in the disk)
+                if lhash:
+                    log_debug("comp yield: file %r is gone!", fullname)
+                    events.append(('FS_FILE_DELETE', fullname))
         return events, to_scan_later
 
-    def _paths_filter(self, all_share_dirs, dirpath, len_shr_path):
+    def _paths_filter(self, shrinfo, dirpath, len_shr_path):
         '''Returns the paths that belong to this dir.'''
         # paths in shares are relative, remove the first slash
         direct = dirpath[len_shr_path + 1:]
@@ -465,18 +467,31 @@ class LocalRescan(object):
 
         # build the dict
         filesdirs = {}
-        for shrpath, is_dir, statinfo, changed, node_id in all_share_dirs:
-            base, fname = os.path.split(shrpath)
+        for fpath, is_dir, statinfo, changed, node_id, lhash, shash in shrinfo:
+            base, fname = os.path.split(fpath)
             if base == direct:
                 # if without node_id, remove the metadata, and take it as new
                 if node_id is None:
-                    fullname = os.path.join(basedir, shrpath)
+                    fullname = os.path.join(basedir, fpath)
                     m = "Deleting metadata, because of node_id=None, of %r"
                     log_debug(m, fullname)
                     self.fsm.delete_metadata(fullname)
-                    continue
 
-                filesdirs[fname] = is_dir, statinfo, changed
+                # if both hashes aren't set in a file, it's a non-content
+                # situation, remove the metadata
+                elif not is_dir and not lhash and not shash:
+                    fullname = os.path.join(basedir, fpath)
+                    m = "Deleting metadata, both hashes empty, of %r"
+                    log_debug(m, fullname)
+                    self.fsm.delete_metadata(fullname)
+
+                    # also set the parent hashes to "", to force a new scan
+                    parent = os.path.dirname(fullname)
+                    log_debug("Dirtying the parent hashes, path: %r", parent)
+                    self.fsm.set_by_path(parent, server_hash="", local_hash="")
+
+                else:
+                    filesdirs[fname] = is_dir, statinfo, changed, lhash
         return filesdirs
 
     def _scan_one_dir(self, scan_info):
