@@ -317,16 +317,7 @@ class SyncStateMachineRunner(StateMachineRunner):
 
     def validate_actual_data(self, path, oldstat):
         """Validates that the received info is not obsolete."""
-        try:
-            newstat = os.stat(path)
-        except OSError:
-            # the file went away between the moment HQ finished and now, we
-            # discard it (if the file would have went away moments before,
-            # this info would have been discarded by HQ itself)
-            m = "Received hash of path, but it's no longer there: %r"
-            self.log.debug(m, path)
-            return False
-
+        newstat = os.stat(path)
         if newstat.st_ino != oldstat.st_ino or \
            newstat.st_size != oldstat.st_size or \
            newstat.st_mtime != oldstat.st_mtime or \
@@ -1027,8 +1018,17 @@ class Sync(object):
         key = FSKey(self.m.fs, path=path)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
-        if ssmr.validate_actual_data(path, stat):
-            ssmr.signal_event_with_hash("HQ_HASH_NEW", hash, crc32, size, stat)
+        try:
+            data_ok = ssmr.validate_actual_data(path, stat)
+        except OSError, e:
+            # the file went away between the moment HQ finished and now, we
+            # discard the info, but needs to send to rehash.
+            log.debug("Changing HQ_HASH_NEW to ERROR in %r: %s", path, e)
+            ssmr.on_event('HQ_HASH_ERROR', {})
+        else:
+            if data_ok:
+                ssmr.signal_event_with_hash("HQ_HASH_NEW", hash,
+                                            crc32, size, stat)
 
     def handle_AQ_UPLOAD_FINISHED(self, share_id, node_id, hash):
         """on AQ_UPLOAD_FINISHED"""
