@@ -178,7 +178,7 @@ class CapabilitiesError(Exception):
 
 class Client(object):
     """U1 storage client facade."""
-    required_caps = frozenset(["no-content"])
+    required_caps = frozenset(["no-content", "fix462230"])
 
     def __init__(self, realm, reactor=reactor):
         """Create the instance."""
